﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using EagleWare.Libraries.FlacCommands;
using EagleWare.Libraries.FLServerCommunications;
using EagleWare.Libraries.FLServerCommunications.Events;
using EagleWare.Libraries.GeneralNonsense.Extensions;
using EagleWare.Libraries.GeneralNonsense.Timers;

namespace FlacPlugIn.Banners {
  internal class Banners: AdminCommand {
    private Timer _timer = null;
    private int _nextBannerIndex = 0;

    public Banners() {
      CreateBannerLoopTimer();
      ParametersPattern =
        @"^\s*((?<Add>((?i)add(?-i)))\s+(?<Style>0x[a-fA-F0-9]{8})\s+(?<Message>.+?)|(?<Append>((?i)append(?-i)))\s+(?<AppendPosition>[1-9]\d?)\s*:(?<AppendMessage>.+?)|(?<Help>((?i)help(?-i)))|(?<Insert>((?i)insert(?-i)))\s+(?<OldPosition>[1-9]\d?)\s+at\s+(?<NewPosition>[1-9]\d?)|(?<List>((?i)list(?-i)))\s+((?<ListAll>((?i)all(?-i)))|(?<ListFrom>[1-9]\d?)\s*-\s*(?<ListTo>[1-9]\d?))|(?<Remove>((?i)remove(?-i)))\s+(?<RemovePosition>[1-9]\d?)|(?<SetDelay>((?i)set\s+delay(?-i)))\s+(?<DelaySeconds>[1-9]\d{0,2})|(?<ShowDelay>((?i)show\s+delay(?-i))))\s*$";
      Syntax += " [add|help|insert|list|remove|set delay|show delay]";
      Description = "Manages banners";
      Usage = GetUsageText();
    }

    private void CreateBannerLoopTimer() {
      if(_timer != null) _timer.Stop();
      _timer = Settings.Instance.TimerFactory.CreateTimer(Settings.Instance.Configuration.InterBannerDelay, DoBannerLoop);
    }

    private void DoBannerLoop() {
      if(Settings.Instance.Configuration.Banners.Count == 0) return;
      lock(this) {
        if(Settings.Instance.Configuration.Banners.Count <= _nextBannerIndex) _nextBannerIndex = 0;
        Banner banner = Settings.Instance.Configuration.Banners[_nextBannerIndex];
        UniverseCustomChat.Fire(banner.Style, banner.Message.Replace("%timestamp%", Settings.Instance.ServerInformation.GetTimeStampString()));
      }
      _nextBannerIndex++;
    }

    public override void Execute(Player user, string parameters) {
      Match match = ParametersRegularExpression.Match(parameters);
      HandleAdd(user, match);
      HandleAppend(user, match);
      HandleHelp(user, match);
      HandleInsert(user, match);
      HandleList(user, match);
      HandleRemove(user, match);
      HandleSetDelay(user, match);
      HandleShowDelay(user, match);
    }

    private void HandleAdd(Player player, Match match) {
      if(match.Groups["Add"].Value.IsEmpty()) return;
      Banner banner = new Banner();
      banner.Style = Convert.ToUInt32(match.Groups["Style"].Value, 16);
      banner.Message = match.Groups["Message"].Value;
      lock(this) {
        Settings.Instance.Configuration.Banners.Add(banner);
        Settings.Instance.Configuration.Save();
      }
      PrivateChat.Fire(player, string.Format("Banner added to position {0} in the list.", Settings.Instance.Configuration.Banners.Count));
    }

    private int GetIndexFromPosition(int position) {
      position = Settings.Instance.Configuration.Banners.Count < position? Settings.Instance.Configuration.Banners.Count: position;
      return position <= 0? 0: position-1;
    }

    private void HandleAppend(Player player, Match match) {
      if(match.Groups["Append"].Value.IsEmpty()) return;
      int position = GetIndexFromPosition(Int32.Parse(match.Groups["AppendPosition"].Value));
      Banner banner = Settings.Instance.Configuration.Banners[position];
      banner.Message += match.Groups["AppendMessage"].Value;
      lock(this) {
        Settings.Instance.Configuration.Banners[position] = banner;
        Settings.Instance.Configuration.Save();
      }
      PrivateChat.Fire(player, string.Format("Text appended to banner {0}.", Settings.Instance.Configuration.Banners.Count));
    }

    private void HandleHelp(Player player, Match match) {
      if(match.Groups["Help"].Value.IsEmpty()) return;
      PrivateChat.Fire(player, GetUsageText());
    }

    private string GetUsageText() {
      return @"/banners add [style] [message]
    Adds a new banner to the end of the banner list.
    Note! %timestamp% in the message will be replaced by date/time.
    Example: /banners add 0xFFFFFF00 Hello world!
/banners append [position]:[text]
    Appends text after the colon to the end of the selected banner.
    Example: /banners append 5: the trailing text.
/banners help
    Shows this help screen.
/banners insert [old position] at [new position]
    Allows you to reorder banners in the list.
    Example: /banners insert 3 at 1
/banners list (all|[from]-[to])
    Lists properties of the requested banners.
    Example 1: /banners list all
    Example 2: /banners list 6 - 10
/banners remove [position]
    Removes banner at provided position from the list.
/banners set delay [period]
    Sets the inter banner delay in seconds.
/banners show delay
    Displays the inter banner delay in seconds.";
    }

    private void HandleInsert(Player player, Match match) {
      if(match.Groups["Insert"].Value.IsEmpty()) return;
      int oldPosition = GetIndexFromPosition(Int32.Parse(match.Groups["OldPosition"].Value));
      int newPosition = GetIndexFromPosition(Int32.Parse(match.Groups["NewPosition"].Value));
      if(newPosition == oldPosition) {
        PrivateChat.Fire(player, "Can't perform insert, aborting...");
        return;
      }
      lock(this) {
        Banner banner = Settings.Instance.Configuration.Banners[oldPosition];
        Settings.Instance.Configuration.Banners.RemoveAt(oldPosition);
        Settings.Instance.Configuration.Banners.Insert(newPosition, banner);
        Settings.Instance.Configuration.Save();
      }
      PrivateChat.Fire(player, string.Format("Moved banner {0} to position {1}", oldPosition+1, newPosition+1));
    }

    private void HandleList(Player player, Match match) {
      if(match.Groups["List"].Value.IsEmpty()) return;
      if(match.Groups["ListAll"].Value.IsEmpty()) {
        ListFromTo(player, match);
        return;
      }
      ListAll(player);
    }

    private void ListAll(Player player) {
      List(player, 1, Settings.Instance.Configuration.Banners.Count);
    }

    private void ListFromTo(Player player, Match match) {
      List(player, Int32.Parse(match.Groups["ListFrom"].Value), Int32.Parse(match.Groups["ListTo"].Value));
    }

    private void List(Player player, int start, int end) {
      if(Settings.Instance.Configuration.Banners.Count == 0) {
        PrivateChat.Fire(player, "Banner list is empty.");
        return;
      }
      start = GetIndexFromPosition(start);
      end = GetIndexFromPosition(end);
      if(end < start) {
        PrivateChat.Fire(player, string.Format("End index {0} is smaller than start {1}, aborting...", end, start));
        return;
      }
      ListIndex(player, start, end);
    }

    private void ListIndex(Player player, int start, int end) {
      List<string> bannerProperties = new List<string>();
      for(int i=start; i<=end; i++) {
        bannerProperties.Add(
          string.Format(
            "{0}. 0x{1} {2}",
            i+1,
            Convert.ToString(Settings.Instance.Configuration.Banners[i].Style, 16).ToUpper().PadLeft(8, '0'),
            Settings.Instance.Configuration.Banners[i].Message
          )
        );
      }
      PrivateChat.Fire(player, string.Join("\n", bannerProperties.ToArray()));
    }

    private void HandleRemove(Player player, Match match) {
      if(match.Groups["Remove"].Value.IsEmpty()) return;
      int removePosition = Int32.Parse(match.Groups["RemovePosition"].Value);
      if(Settings.Instance.Configuration.Banners.Count < removePosition) {
        PrivateChat.Fire(player, "Position exceeds number of banners in the list, aborting...");
        return;
      }
      lock(this) {
        removePosition = GetIndexFromPosition(removePosition);
        PrivateChat.Fire(player, string.Format("Removing banner {0}: {1}", removePosition+1, Settings.Instance.Configuration.Banners[removePosition].Message));
        Settings.Instance.Configuration.Banners.RemoveAt(removePosition);
        Settings.Instance.Configuration.Save();
      }
    }

    private void HandleSetDelay(Player player, Match match) {
      if(match.Groups["SetDelay"].Value.IsEmpty()) return;
      uint delaySeconds = UInt32.Parse(match.Groups["DelaySeconds"].Value);
      if(Settings.Instance.Configuration.InterBannerDelay != delaySeconds) {
        Settings.Instance.Configuration.InterBannerDelay = delaySeconds;
        Settings.Instance.Configuration.Save();
        CreateBannerLoopTimer();
      }
      DisplayInterBannerDelay(player);
    }

    private void HandleShowDelay(Player player, Match match) {
      if(match.Groups["ShowDelay"].Value.IsEmpty()) return;
      DisplayInterBannerDelay(player);
    }

    private static void DisplayInterBannerDelay(Player player) {
      PrivateChat.Fire(player, string.Format("Inter banner delay set at {0} seconds.", Settings.Instance.Configuration.InterBannerDelay));
    }
  }
}
