﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EagleWare.Libraries.GeneralNonsense;

namespace FlacPlugIn.DynamicEconomy {
  internal abstract class Bases {
    private Dictionary<uint, Base> _bases;

    protected abstract class Base {
      protected Logger _log = Utilities.Instance.Log;
      private Dictionary<uint, Commodity> _commodities;
      private List<uint> _linkedBases;

      public Base(uint baseID) {
        BaseID = baseID;
        _commodities = new Dictionary<uint, Commodity>();
        _linkedBases = new List<uint>();
        _linkedBases.Add(BaseID);
      }

      public uint BaseID { get; private set; }

      public void LinkBase(uint baseID) {
        if(_linkedBases.Contains(baseID)) return;
        _linkedBases.Add(baseID);
        foreach(Commodity commodity in _commodities.Values) {
          commodity.LinkBase(baseID);
        }
      }

      protected void AddCommodity(Commodity commodity) {
        _commodities.Add(commodity.GoodID, commodity);

        foreach(uint baseID in _linkedBases) {
          commodity.LinkBase(baseID);
        }
      }

      public Commodity GetCommodity(uint goodID) {
        return _commodities.ContainsKey(goodID)? _commodities[goodID]: null;
      }

      public void ProcessTrade(uint goodID, int count, bool removeFromStock) {
        Commodity commodity = GetCommodity(goodID);
        if(commodity == null) return;
        count *= removeFromStock? -1: 1;
        _log.Info(
          "Processing trade on base {0} of commodity {1}, count: {2}",
          Utilities.Instance.GetDescription(BaseID),
          Utilities.Instance.GetDescription(goodID),
          count
        );
        commodity.InStock += count;
      }

      public void InitializeServer() {
        foreach(Commodity commodity in _commodities.Values) {
          commodity.InitializeServer();
        }
      }

      public void DoTimeSliceUpdate() {
        foreach(Commodity commodity in _commodities.Values) {
          commodity.DoTimeSliceUpdate();
        }
      }
    }

    public Bases() {
      _bases = new Dictionary<uint, Base>();
    }

    protected Dictionary<uint, Base>.ValueCollection BaseCollection { get { return _bases.Values; } }

    public void InitializeServer() {
      foreach(Base thisBase in _bases.Values) {
        thisBase.InitializeServer();
      }
    }

    public void DoTimeSliceUpdate() {
      foreach(uint baseID in _bases.Keys) {
        if(_bases[baseID].BaseID != baseID) continue;

        _bases[baseID].DoTimeSliceUpdate();
      }
    }

    protected void AddBase(Base thisBase) {
      if(_bases.ContainsKey(thisBase.BaseID)) return;

      _bases[thisBase.BaseID] = thisBase;
      foreach(uint baseID in BaseManager.Instance.GetLinkedBases(thisBase.BaseID)) {
        LinkBase(thisBase, baseID);
      }
    }

    protected Base GetBase(uint baseID) {
      return _bases.ContainsKey(baseID)? _bases[baseID]: null;
    }

    private void LinkBase(Base thisBase, uint baseID) {
      if(_bases.ContainsKey(baseID)) return;

      thisBase.LinkBase(baseID);
      _bases[baseID] = thisBase;
    }

    public void ProcessTrade(uint baseID, uint goodID, int count, bool removeFromStock) {
      if(!_bases.ContainsKey(baseID)) return;
      _bases[baseID].ProcessTrade(goodID, count, removeFromStock);
    }

    public IEnumerable<uint> GetDealers(uint goodID) {
      return
        from h in _bases.Keys
        where _bases[h].GetCommodity(goodID) != null
        select h;
    }

    public bool Contains(uint baseID, uint goodID) {
      return _bases.ContainsKey(baseID) && _bases[baseID].GetCommodity(goodID) != null;
    }
  }
}
