﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EagleWare.Libraries.GeneralNonsense;
using EagleWare.Libraries.FLServerCommunications;

namespace FlacPlugIn.DynamicEconomy {
  internal abstract class Commodity {
    protected Logger _log = Utilities.Instance.Log;
    private int _inStock;
    private Dictionary<uint, ServerCommodity> _linkedBases;

    private class ServerCommodity {
      private Logger _log = Utilities.Instance.Log;
      private uint _baseID = 0;
      private uint _goodID = 0;
      private float _price = 0;
      private bool _sellCommodity = false;
      private bool _modified = false;
      private int _updateSliceDelay = 0;

      public ServerCommodity() {
        ReSet();
      }

      private int UpdataSliceDelay {
        get { return _updateSliceDelay; }
        set {
          if(value < 0) {
            _updateSliceDelay = 0;
            return;
          }
          _updateSliceDelay = value;
        }
      }

      private bool Modified {
        get { return _modified && UpdataSliceDelay == 0; }
        set {
          if(!value) {
            _modified = false;
            return;
          }
          if(_modified) return;
          _modified = true;
          UpdataSliceDelay = 3;
        }
      }

      public uint BaseID {
        get { return _baseID; }
        private set {
          if(value == BaseID) return;
          _baseID = value;
          Modified = true;
        }
      }

      public uint GoodID {
        get { return _goodID; }
        private set {
          if(value == GoodID) return;
          _goodID = value;
          Modified = true;
        }
      }

      public float Price {
        get { return _price; }
        private set {
          if(Math.Floor(value) == Math.Floor(Price)) return;
          _price = value;
          Modified = true;
        }
      }

      public bool SellCommodity {
        get { return _sellCommodity; }
        private set {
          if(value == SellCommodity) return;
          _sellCommodity = value;
          Modified = true;
        }
      }

      public void ReSet() {
        Modified = true;
        UpdataSliceDelay = 0;
      }

      public void UpdateServer(uint baseID, uint goodID, float price, bool sellCommodity) {
        BaseID = baseID;
        GoodID = goodID;
        Price = price;
        SellCommodity = sellCommodity;
        UpdataSliceDelay--;
        if(!Modified) return;
        Modified = false;

        _log.Debug(
          "Updating server commodity: {0}, base: {1}, price: {2}",
          Utilities.Instance.GetDescription(GoodID),
          Utilities.Instance.GetDescription(BaseID),
          Price
        );
        BaseManager.Instance.AddServerCommodity(
          baseID, new BaseCommodity(GoodID) { Price = Price, IsSold = SellCommodity }
        );
      }
    }

    public Commodity(uint baseID, uint goodID) {
      BaseID = baseID;
      GoodID = goodID;
      _linkedBases = new Dictionary<uint, ServerCommodity>();
      _linkedBases.Add(BaseID, new ServerCommodity());
      StockCapacity = Utilities.Instance.NominalCapacity;
    }

    public uint BaseID { get; private set; }

    public uint GoodID { get; private set; }

    public int StockCapacity { get; set; }

    public int InStock {
      get { return _inStock; }
      set { _inStock = value < 0? 0: (StockCapacity < value? StockCapacity: value); }
    }

    protected float PriceFactor { get { return Utilities.Instance.GetPriceFactor(InStock); } }

    public abstract float StorePrice { get; }

    protected abstract bool SellCommodity { get; }

    public void LinkBase(uint baseID) {
      if(_linkedBases.ContainsKey(baseID)) return;
      _linkedBases.Add(baseID, new ServerCommodity());
    }

    public void InitializeServer() {
      foreach(ServerCommodity commodity in _linkedBases.Values) {
        commodity.ReSet();
      }
    }

    private void SetServerCommodity() {
      foreach(uint baseID in _linkedBases.Keys) {
        _linkedBases[baseID].UpdateServer(baseID, GoodID, StorePrice, SellCommodity);
      }
    }

    public void DoTimeSliceUpdate() {
      Update();
      SetServerCommodity();
    }

    protected abstract void Update();
  }
}
