﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml;
using System.Globalization;
using EagleWare.Libraries.GeneralNonsense;
using EagleWare.Libraries.IniProcessor;

namespace FlacPlugIn.DynamicEconomy {
  internal class MarketCommoditiesParser {
    private class Commodity {
      public Commodity(uint hash, float value, bool buyOnly) {
        Hash = hash;
        Value = value;
        BuyOnly = buyOnly;
      }

      public uint Hash { get; private set; }

      public float Value { get; private set; }

      public bool BuyOnly { get; private set; }
    }

    private Logger _log = Utilities.Instance.Log;
    private IniParser _commodityParser = null;
    private IniParser _goodsParser = null;

    public MarketCommoditiesParser() { }

    public void CreateDynamicCommodityMarket() {
      CreateLinkedBases(Utilities.Instance.ConfigurationNode);
      CreateDynamicMarket(Utilities.Instance.ConfigurationNode);
    }

    private void CreateLinkedBases(XmlElement configuration) {
      foreach(XmlElement baseNode in configuration.SelectNodes("linkedbases/base")) {
        string mainBase = baseNode.GetAttribute("name");
        foreach(XmlElement linkNode in baseNode.SelectNodes("linkto")) {
          string linkBase = linkNode.GetAttribute("name");
          _log.Info("Linking bases: {0} and {1}", mainBase, linkBase);
          BaseManager.Instance.LinkBase(mainBase, linkBase);
        }
      }
    }

    private void CreateDynamicMarket(XmlElement configuration) {
      _log.Info("Creating dynamic market");

      foreach(XmlElement commodityNode in configuration.SelectNodes("commodities/commodity")) {
        string commodity = commodityNode.GetAttribute("name");
        _log.Info("Processing commodity: {0}", commodity);
        uint goodID = Utilities.Instance.GetHash(commodity);
        AddProducers(goodID, commodityNode.SelectSingleNode("producers") as XmlElement);
        AddResellers(goodID, commodityNode.SelectSingleNode("resellers") as XmlElement);
        AddConsumers(goodID);
      }
    }

    private float GetGoodsPrice(Section section) {
      foreach(Parameter parameter in section.Parameters) {
        if(parameter.Key.ToLower() != "price") continue;
        return Convert.ToSingle(parameter.Value, NumberFormatInfo.InvariantInfo);
      }
      return 0;
    }

    private void AddProducers(uint goodID, XmlElement producers) {
      var values = GetCommodityValues(goodID);
      float price = GetGoodsPrice(GetGoodsSection(goodID));
      foreach(XmlElement element in producers.SelectNodes("producer")) {
        string producer = element.GetAttribute("name");
        uint baseID = Utilities.Instance.GetHash(producer);
        if(!values.ContainsKey(baseID)) {
          _log.Info("Can't find market data for producer {0}, skipping...", producer);
          continue;
        }

        Commodity commodity = values[baseID];
        _log.Info("Adding producer base {0}", producer);
        // 1/Utilities.PriceFactor
        BaseManager.Instance.Producers.AddProducerCommodity(
          baseID,
          commodity.Hash,
          0.8F * commodity.Value * price,
          1/commodity.Value
        );
      }
    }

    private void AddResellers(uint goodID, XmlElement resellers) {
      var values = GetCommodityValues(goodID);
      float price = GetGoodsPrice(GetGoodsSection(goodID));
      foreach(XmlElement element in resellers.SelectNodes("reseller")) {
        string reseller = element.GetAttribute("name");
        uint baseID = Utilities.Instance.GetHash(reseller);
        if(!values.ContainsKey(baseID)) {
          _log.Info("Can't find market data for reseller {0}, skipping...", reseller);
          continue;
        }

        Commodity commodity = values[baseID];
        _log.Info("Adding reseller base {0}", reseller);
        // 1/Utilities.PriceFactor
        BaseManager.Instance.Resellers.AddResellerCommodity(baseID, commodity.Hash, 0.399580F * commodity.Value, price);
      }
    }

    private void AddConsumers(uint goodID) {
      var values = GetCommodityValues(goodID);
      float price = GetGoodsPrice(GetGoodsSection(goodID));
      foreach(uint baseID in values.Keys) {
        if(BaseManager.Instance.Producers.Contains(baseID, goodID)) continue;
        if(BaseManager.Instance.Resellers.Contains(baseID, goodID)) continue;

        Commodity commodity = values[baseID];
        _log.Info("Adding consumer: base {0}", baseID);
        // 1/Utilities.PriceFactor
        BaseManager.Instance.Consumers.AddConsumerCommodity(baseID, commodity.Hash, 0.333333F * commodity.Value, price);
      }
    }

    private Section GetGoodsSection(uint goodID) {
      foreach(Section section in GoodsParser.Sections) {
        if(Utilities.Instance.GetHash(section.NickName) == goodID) return section;
      }
      _log.Info("Not found good: {0}", Utilities.Instance.GetDescription(goodID));
      return null;
    }

    private Dictionary<uint, Commodity> GetCommodityValues(uint goodID) {
      var values = new Dictionary<uint, Commodity>();
      foreach(Section section in CommodityParser.Sections) {
        if(section.Header.ToLower() != "basegood") continue;
        ParseCommoditySection(section, goodID, values);
      }
      return values;
    }

    private void ParseCommoditySection(Section section, uint goodID, Dictionary<uint, Commodity> values) {
      uint baseID = 0;
      float itemValue = 1;
      bool isBuyOnly = true;
      foreach(Parameter parameter in section.Parameters) {
        if(parameter.Key.ToLower() == "base") baseID = Utilities.Instance.GetHash(parameter.Value.Trim());
        if(parameter.Key.ToLower() != "marketgood") continue;
        string[] paramValues = parameter.Value.Split(',');
        if(Utilities.Instance.GetHash(paramValues.First().Trim()) != goodID) continue;
        itemValue = Convert.ToSingle(paramValues.Last(), NumberFormatInfo.InvariantInfo);
        isBuyOnly = Convert.ToInt32(paramValues[3]) == 0;
      }
      if(baseID == 0) return;
      values.Add(baseID, new Commodity(goodID, itemValue, isBuyOnly));
    }

    private IniParser CommodityParser {
      get { return _commodityParser == null? _commodityParser = CreateCommodityParser(): _commodityParser; }
    }

    private IniParser GoodsParser {
      get { return _goodsParser == null? _goodsParser = CreateGoodsParser(): _goodsParser; }
    }

    private IniParser CreateCommodityParser() {
      return new IniParser(Path.Combine(GetFreelancerRoot(), @"DATA\EQUIPMENT\market_commodities.ini"));
    }

    private IniParser CreateGoodsParser() {
      return new IniParser(Path.Combine(GetFreelancerRoot(), @"DATA\EQUIPMENT\goods.ini"));
    }

    private string GetFreelancerRoot() {
      Microsoft.Win32.RegistryKey regKey = Microsoft.Win32.Registry.LocalMachine;
      regKey = regKey.OpenSubKey(@"Software\Microsoft\Microsoft Games\Freelancer\1.0");
      return regKey.GetValue("AppPath", "").ToString();
    }
  }
}
