﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace FlacPlugIn.DynamicEconomy {
  internal class ResellerCommodity: Commodity {
    private class Order {
      public Order(uint baseID, uint ordererID, uint goodID, int deliveryTime, int orderSize) {
        BaseID = baseID;
        DeliveryTime = deliveryTime;
        OrderSize = orderSize;
        if(OrderSize == 0) return;
        Utilities.Instance.Log.Info("Processing order from base: {0}", ordererID);
        BaseManager.Instance.Producers.ProcessTrade(baseID, goodID, orderSize, true);
      }

      public uint BaseID { get; private set; }

      public int DeliveryTime { get; set; }

      public int OrderSize { get; private set; }
    }

    private Queue<Order> _orders;

    public ResellerCommodity(uint baseID, uint goodID, float localeFactor, float basePrice)
      : base(baseID, goodID) {
      InStock = Utilities.Instance.NominalCapacity/10;
      StockOrderThreshold = Utilities.Instance.NominalCapacity/50;
      MinimumOrder = Utilities.Instance.NominalCapacity/5;
      ConsumptionRate = 7;
      DeliveryTime = 5;
      InitialStock = InStock;
      LocaleFactor = localeFactor;
      _orders = new Queue<Order>();
      BasePrice = basePrice;
    }

    private float BasePrice { get; set; }

    protected override bool SellCommodity { get { return 0 < InStock; } }

    public int ConsumptionRate { get; set; }

    private int InitialStock { get; set; }

    public int StockOrderThreshold { get; set; }

    public int MinimumOrder { get; set; }

    public int DeliveryTime { get; set; }

    public float LocaleFactor { get; private set; }

    private int Demand { get { return (InitialStock - InStock) * DeliveryTime; } }

    private int EffectiveStock {
      get {
        return InStock + _orders.Sum(
          delegate(Order order) {
            return order.OrderSize;
          }
        );
      }
    }

    private int OrderTotal { get { return Math.Min(Convert.ToInt32(1.05 * Demand) - EffectiveStock, StockCapacity/3); } }

    private int ForceOrderTotal { get { return EffectiveStock < StockOrderThreshold? MinimumOrder - EffectiveStock: 0; } }

    private int EffectiveOrder { get { return Math.Max(OrderTotal, ForceOrderTotal); } }

    public override float StorePrice {
      get {
        float storePrice = LocaleFactor * BasePrice * PriceFactor + (SupplierPrice  - BasePrice);
        return storePrice < 1? 1: storePrice;
      }
    }

    private float SupplierPrice { get { return BaseManager.Instance.Producers.GetSupplierPrice(GoodID); } }

    private void UpdateOrderDeliveryTimes() {
      foreach(Order order in _orders) {
        order.DeliveryTime -= 1;
      }
    }

    protected override void Update() {
      UpdateOrderDeliveryTimes();
      OrderNewStock();
      ProcessBackOrders();
      if(InitialStock < InStock) InStock -= ConsumptionRate;
    }

    private void OrderNewStock() {
      _orders.Enqueue(new Order(
        BaseManager.Instance.Producers.GetSupplierBase(GoodID),
        BaseID,
        GoodID,
        DeliveryTime,
        Math.Min(BaseManager.Instance.Producers.GetSupplierStock(GoodID), EffectiveOrder)
      ));
    }

    private void ProcessBackOrders() {
      if(0 < _orders.Peek().DeliveryTime) return;
      Order order = _orders.Dequeue();
      if(order.OrderSize <= 0) return;
      int oldStock = InStock;
      InStock += order.OrderSize;
      if(oldStock == InStock) return;
    }
  }
}
